"""Test the module result."""

from __future__ import annotations

import pytest

from pytest_ansible.results import ModuleResult


positive_host_patterns = {
    "all": 2,
    "*": 2,
    "localhost": 1,
    "local*": 1,
    "local*:&*host": 1,
    "!localhost": 1,
    "all[0]": 1,
    "all[-1]": 1,
    "*[0:1]": 2,  # this is confusing, but how host slicing works
    "*[0:]": 2,
}
negative_host_patterns = {
    "none": 0,
    "all[8:]": 0,
}

valid_hosts = ("localhost", "another_host")
invalid_hosts = ("none", "all", "*", "local*")


@pytest.fixture()
def module_result_ok(request):  # type: ignore[no-untyped-def]  # noqa: ANN001, ANN201, ARG001, D103
    return ModuleResult(
        invocation={"module_name": "debug", "module_args": {"msg": "testing"}},
        msg="testing",
        changed=False,
        _ansible_verbose_always=True,
        _ansible_no_log=False,
    )


@pytest.fixture()
def module_result_failed():  # type: ignore[no-untyped-def]  # noqa: ANN201, D103
    return ModuleResult(
        invocation={"module_name": "fail", "module_args": {}},
        failed=True,
        changed=False,
        _ansible_no_log=False,
        msg="Failed as requested from task",
    )


@pytest.fixture()
def module_result_changed(request):  # type: ignore[no-untyped-def]  # noqa: ANN001, ANN201, ARG001, D103
    return ModuleResult(
        changed=True,
        end="2016-06-17 21:32:54.877597",
        _ansible_no_log=False,
        stdout="Fri Jun 17 21:32:54 EDT 2016",
        cmd=["date"],
        rc=0,
        start="2016-06-17 21:32:54.873429",
        stderr="",
        delta="0:00:00.004168",
        invocation={
            "module_name": "command",
            "module_args": {
                "creates": None,
                "executable": None,
                "_uses_shell": False,
                "_raw_params": "date",
                "removes": None,
                "warn": True,
                "chdir": None,
            },
        },
        stdout_lines=["Fri Jun 17 21:32:54 EDT 2016"],
        warnings=[],
    )


@pytest.fixture()
def _module_result_skipped():  # type: ignore[no-untyped-def]  # noqa: ANN202
    msg = "Coming soon!"
    raise NotImplementedError(msg)


@pytest.fixture()
def _module_result_unreachable():  # type: ignore[no-untyped-def]  # noqa: ANN202
    msg = "Coming soon!"
    raise NotImplementedError(msg)


@pytest.mark.parametrize(
    ("fixture_name", "prop", "expected_result"),
    (
        ("module_result_ok", "is_ok", True),
        ("module_result_ok", "is_successful", True),
        ("module_result_failed", "is_failed", True),
        ("module_result_failed", "is_successful", False),
        ("module_result_changed", "is_changed", True),
        ("module_result_changed", "is_successful", True),
        pytest.param(
            "module_result_skipped",
            "is_skipped",
            True,
            marks=pytest.mark.skipif("True"),
        ),
        pytest.param(
            "module_result_unreachable",
            "is_unreachable",
            True,
            marks=pytest.mark.skipif("True"),
        ),
    ),
)
def test_is_property(request, fixture_name, prop, expected_result):  # type: ignore[no-untyped-def]  # noqa: ANN001, ANN201, D103
    fixture = request.getfixturevalue(fixture_name)
    assert getattr(fixture, prop) == expected_result



"""pytest-molecule plugin implementation."""

from __future__ import annotations

import importlib.util
import logging
import os
import shlex
import subprocess
import sys
import warnings

from importlib.metadata import version
from pathlib import Path

import pytest
import yaml

from ansible_compat.config import ansible_version


# Do not add molecule imports here as it does have side effects due to console
# redirection. We need to do these as lazy as possible.


molecule_spec = importlib.util.find_spec("molecule")
HAS_MOLECULE = molecule_spec is not None


logger = logging.getLogger(__name__)
counter = 0


def molecule_pytest_configure(config):  # type: ignore[no-untyped-def]  # noqa: ANN001, ANN201
    """Pytest hook for loading our specific configuration."""
    interesting_env_vars = [
        "ANSIBLE",
        "MOLECULE",
        "DOCKER",
        "PODMAN",
        "VAGRANT",
        "VIRSH",
        "ZUUL",
    ]

    # Add extra information that may be key for debugging failures
    if hasattr(config, "_metadata"):
        for package in ["molecule"]:
            config._metadata["Packages"][package] = version(  # noqa: SLF001
                package,
            )

        if "Tools" not in config._metadata:  # noqa: SLF001
            config._metadata["Tools"] = {}  # noqa: SLF001
        config._metadata["Tools"]["ansible"] = str(ansible_version())  # noqa: SLF001

        # Adds interesting env vars
        env = ""
        for key, value in sorted(os.environ.items()):
            for var_name in interesting_env_vars:
                if key.startswith(var_name):
                    env += f"{key}={value} "
        config._metadata["env"] = env  # noqa: SLF001

    # We hide DeprecationWarnings thrown by driver loading because these are
    # outside our control and worse: they are displayed even on projects that
    # have no molecule tests at all as pytest_configure() is called during
    # collection, causing spam.
    with warnings.catch_warnings():
        warnings.filterwarnings("ignore", category=DeprecationWarning)

        config.option.molecule = {}
        from molecule.api import drivers

        for driver in map(str, drivers()):
            config.addinivalue_line(
                "markers",
                f"{driver}: mark test to run only when {driver} is available",
            )
            config.option.molecule[driver] = {"available": True}

        config.addinivalue_line(
            "markers",
            "no_driver: mark used for scenarios that do not contain driver info",
        )

        config.addinivalue_line(
            "markers",
            "molecule: mark used by all molecule scenarios",
        )

        # validate selinux availability
        if sys.platform == "linux" and Path("/etc/selinux/config").is_file():
            try:
                import selinux  # noqa: F401 pylint: disable=import-outside-toplevel
            except ImportError:
                logging.exception(
                    "It appears that you are trying to use "
                    "molecule with a Python interpreter that does not have the "
                    "libselinux python bindings installed. These can only be "
                    "installed using your distro package manager and are specific "
                    "to each python version. Common package names: "
                    "libselinux-python python2-libselinux python3-libselinux",
                )
                # we do not re-raise this exception because missing or broken
                # selinux bindings are not guaranteed to fail molecule execution.


class MoleculeFile(pytest.File):
    """Wrapper class for molecule files."""

    def collect(self):  # type: ignore[no-untyped-def]  # noqa: ANN201
        """Test generator."""
        # pylint: disable=global-statement
        global counter  # noqa: PLW0603
        if hasattr(MoleculeItem, "from_parent"):
            yield MoleculeItem.from_parent(name=f"test{counter}", parent=self)
        else:
            yield MoleculeItem(f"test{counter}", self)
        counter += 1

    def __str__(self) -> str:
        """Return test name string representation."""
        return str(self.path.relative_to(Path.cwd()))


class MoleculeItem(pytest.Item):
    """A molecule test.

    Pytest supports multiple tests per file, molecule only one "test".
    """

    def __init__(self, name, parent) -> None:  # type: ignore[no-untyped-def]  # noqa: ANN001
        """Construct MoleculeItem."""
        self.funcargs = {}  # type: ignore[var-annotated]
        super().__init__(name, parent)

        # Determine molecule scenario
        scenario_molecule_yml = self.path
        data_scenario = self.yaml_loader(scenario_molecule_yml)  # type: ignore[arg-type]
        # check if there is a global molecule config
        try:
            data_global = self.yaml_loader(
                Path(Path.cwd()) / ".config/molecule/config.yml",  # type: ignore[arg-type]
            )
            data = data_global | data_scenario
        except FileNotFoundError:
            data = data_scenario

        # we add the driver as mark
        self.molecule_driver = data.get("driver", {}).get("name", "no_driver")
        self.add_marker(self.molecule_driver)

        # check for known markers and add them
        markers = data.get("markers", [])
        if "xfail" in markers:
            self.add_marker(
                pytest.mark.xfail(
                    reason="Marked as broken by scenario configuration.",
                ),
            )
        if "skip" in markers:
            self.add_marker(
                pytest.mark.skip(reason="Disabled by scenario configuration."),
            )

        # we also add platforms as marks
        for platform in data.get("platforms", []):
            platform_name = platform["name"]
            self.config.addinivalue_line(
                "markers",
                f"{platform_name}: molecule platform name is {platform_name}",
            )
            self.add_marker(platform_name)
        self.add_marker("molecule")
        if (
            self.config.option.molecule_unavailable_driver
            and not self.config.option.molecule[self.molecule_driver]["available"]
        ):
            self.add_marker(self.config.option.molecule_unavailable_driver)

    def yaml_loader(self, filepath: str) -> dict:  # type: ignore[type-arg]
        """Load a yaml file at a given filepath."""
        with Path.open(filepath, encoding="utf-8") as file_descriptor:  # type: ignore[call-overload]
            return yaml.safe_load(file_descriptor) or {}

    def runtest(self):  # type: ignore[no-untyped-def]  # noqa: ANN201
        """Perform effective test run."""
        folder = self.path.parent
        folders = folder.parts
        cwd = (Path(folder) / "../..").resolve()
        scenario = folders[-1]

        cmd = [sys.executable, "-m", "molecule"]
        if self.config.option.molecule_base_config:
            cmd.extend(("--base-config", self.config.option.molecule_base_config))
        if self.config.option.skip_no_git_change:
            try:
                with subprocess.Popen(  # noqa: S603
                    [  # noqa: S607
                        "git",
                        "diff",
                        self.config.option.skip_no_git_change,
                        "--",
                        "./",
                    ],
                    cwd=cwd,
                    stdout=subprocess.PIPE,
                    stderr=subprocess.STDOUT,
                    universal_newlines=True,
                ) as proc:
                    proc.wait()
                    if len(proc.stdout.readlines()) == 0:  # type: ignore[union-attr]
                        pytest.skip("No change in role")
            except subprocess.CalledProcessError as exc:
                pytest.fail(
                    "Error checking git diff. Error code was: "
                    + str(exc.returncode)
                    + "\nError output was: "
                    + exc.output,
                )

        cmd.extend(("test", "-s", scenario))
        # We append the additional options to molecule call, allowing user to
        # control how molecule is called by pytest-molecule
        opts = os.environ.get("MOLECULE_OPTS")
        if opts:
            cmd.extend(shlex.split(opts))

        if self.config.getoption("--molecule"):  # Check if --molecule option is enabled
            try:
                # Workaround for STDOUT/STDERR line ordering issue:
                # https://github.com/pytest-dev/pytest/issues/5449
                with subprocess.Popen(  # noqa: S603
                    cmd,
                    cwd=cwd,
                    stdout=subprocess.PIPE,
                    stderr=subprocess.STDOUT,
                    universal_newlines=True,
                ) as proc:
                    for line in proc.stdout:  # type: ignore[union-attr]
                        print(line, end="")  # noqa: T201
                    proc.wait()
                    if proc.returncode != 0:
                        pytest.fail(
                            f"Error code {proc.returncode} returned by: {' '.join(cmd)}",
                            pytrace=False,
                        )
            except subprocess.CalledProcessError as exc:
                pytest.fail(
                    f"Exception {exc} returned by: {' '.join(cmd)}",
                    pytrace=False,
                )
        else:
            pytest.skip(
                "Molecule tests are disabled",
            )  # Skip the test if --molecule option is not enabled

    def reportinfo(self):  # type: ignore[no-untyped-def]  # noqa: ANN201
        """Return representation of test location when in verbose mode."""
        return self.fspath, 0, f"use_case: {self.name}"

    def __str__(self) -> str:
        """Return name of the test."""
        return f"{self.name}[{self.molecule_driver}]"


class MoleculeExceptionError(Exception):
    """Custom exception for error reporting."""


class MoleculeScenario:
    """Molecule subprocess wrapper."""

    def __init__(self, name: str, parent_directory: Path, test_id: str) -> None:
        """Initialize the MoleculeScenario class.

        :param molecule_parent: The parent directory of 'molecule'
        :param scenario_name: The name of the molecule scenario
        :param test_id: The test id
        """
        self.parent_directory = parent_directory
        self.name = name
        self.test_id = test_id

    def test(self) -> subprocess.CompletedProcess:  # type: ignore[type-arg]
        """Run molecule test for the scenario.

        :returns: The completed process
        """
        return subprocess.run(
            args=[sys.executable, "-m", "molecule", "test", "-s", self.name],
            capture_output=False,
            check=False,
            cwd=self.parent_directory,
            shell=False,
            text=True,
        )



"""Basic unit tests."""

from __future__ import annotations

import logging
import re
import subprocess
import sys

from typing import TYPE_CHECKING

from pytest_ansible.units import inject, inject_only


if TYPE_CHECKING:
    from pathlib import Path

    import pytest


def test_inject(
    monkeypatch: pytest.MonkeyPatch,
    tmp_path: Path,
    caplog: pytest.LogCaptureFixture,
) -> None:
    """Test injecting a path.

    :param monkeypatch: The pytest monkeypatch fixture
    :param tmp_path: The pytest tmp_path fixture
    :param caplog: The pytest caplog fixture
    """
    caplog.set_level(logging.DEBUG)

    def mock_get_collection_name(start_path: str) -> tuple[str, str]:  # noqa: ARG001
        """Mock the get_collection_name function.

        :param start_path: The path to the root of the collection
        :returns: A tuple of the namespace and name
        """
        return "namespace", "name"

    monkeypatch.setattr(
        "pytest_ansible.units.get_collection_name",
        mock_get_collection_name,
    )

    (tmp_path / "collections" / "ansible_collections").mkdir(parents=True)

    inject(tmp_path)
    assert (tmp_path / "collections" / "ansible_collections" / "namespace" / "name").is_dir()
    assert (
        str(tmp_path / "collections")
        == sys.path[0]
        == re.search(r"_ACF installed: \['(.*?)'.*]", caplog.text).groups()[0]  # type: ignore[union-attr]
        == re.search(r"_ACF configured paths: \['(.*?)'.*]", caplog.text).groups()[0]  # type: ignore[union-attr]
    )


def test_inject_only(
    monkeypatch: pytest.MonkeyPatch,
    tmp_path: Path,
    caplog: pytest.LogCaptureFixture,
) -> None:
    """Test only injecting a path.

    :param monkeypatch: The pytest monkeypatch fixture
    :param tmp_path: The pytest tmp_path fixture
    :param caplog: The pytest caplog fixture
    """
    caplog.set_level(logging.DEBUG)
    monkeypatch.setenv("ANSIBLE_COLLECTIONS_PATH", str(tmp_path / "collections"))

    (tmp_path / "collections" / "ansible_collections").mkdir(parents=True)

    inject_only()
    assert (
        str(tmp_path / "collections")
        == sys.path[0]
        == re.search(r"_ACF installed: \['(.*?)'.*]", caplog.text).groups()[0]  # type: ignore[union-attr]
        == re.search(r"_ACF configured paths: \['(.*?)'.*]", caplog.text).groups()[0]  # type: ignore[union-attr]
    )


def test_for_params():  # type: ignore[no-untyped-def]  # noqa: ANN201
    """Test for params."""
    proc = subprocess.run(
        "pytest --help",  # noqa: S607
        shell=True,
        capture_output=True,
        check=False,
    )
    assert "--ansible-unit-inject-only" in proc.stdout.decode()



"""Fixme."""

from __future__ import annotations

import ansible


class BaseHostManager:
    """The BaseHostManager class provides a base class for managing ansible inventory hosts.

    It's getitem method invokes the module dispatcher with a plugin name.

    Attributes:
        _required_kwargs: A tuple of required keyword arguments.

    """

    _required_kwargs = ("inventory",)

    def __init__(self, *args, **kwargs) -> None:  # type: ignore[no-untyped-def]  # noqa: ANN002, ANN003, ARG002
        """Fixme."""
        self.options = kwargs

        self.check_required_kwargs(**kwargs)  # type: ignore[no-untyped-call]

        # Sub-classes should override this value
        self._dispatcher = self._default_dispatcher

        # Initialize ansible inventory manager
        self.initialize_inventory()  # type: ignore[no-untyped-call]

    def _default_dispatcher(self, **kwargs):  # type: ignore[no-untyped-def]  # noqa: ANN003, ANN202
        pass

    def get_extra_inventory_hosts(self, host_pattern=None):  # type: ignore[no-untyped-def]  # noqa: ANN001, ANN201
        """Fixme."""
        try:
            if host_pattern is None:
                extra_inventory_hosts = [
                    h.name for h in self.options["extra_inventory_manager"].list_hosts()
                ]
            else:
                extra_inventory_hosts = [
                    h.name
                    for h in self.options["extra_inventory_manager"].list_hosts(
                        host_pattern,
                    )
                ]
        except KeyError:
            extra_inventory_hosts = []
        return extra_inventory_hosts

    def get_extra_inventory_groups(self):  # type: ignore[no-untyped-def]  # noqa: ANN201
        """Fixme."""
        try:
            extra_inventory_groups = self.options["extra_inventory_manager"].groups
        except KeyError:
            extra_inventory_groups = []
        return extra_inventory_groups

    def check_required_kwargs(self, **kwargs):  # type: ignore[no-untyped-def]  # noqa: ANN003, ANN201, ARG002
        """Raise a TypeError if any required kwargs are missing."""
        for kwarg in self._required_kwargs:
            if kwarg not in self.options:
                msg = f"Missing required keyword argument '{kwarg}'"
                raise TypeError(msg)

    def has_matching_inventory(self, host_pattern):  # type: ignore[no-untyped-def]  # noqa: ANN001, ANN201
        """Return whether any matching ansible inventory is found for the provided host_pattern."""
        try:
            return (
                len(self.options["inventory_manager"].list_hosts(host_pattern)) > 0
                or host_pattern in self.options["inventory_manager"].groups
                or len(self.get_extra_inventory_hosts(host_pattern)) > 0  # type: ignore[no-untyped-call]
                or host_pattern in self.get_extra_inventory_groups()  # type: ignore[no-untyped-call]
            )
        except ansible.errors.AnsibleError:
            return False

    def __getitem__(self, item):  # type: ignore[no-untyped-def]  # noqa: ANN001, ANN204
        """Return a ModuleDispatcher instance described the provided `item`."""
        # Handle slicing
        if isinstance(item, slice):
            new_item = "all["
            if item.start is not None:
                new_item += str(item.start)
            new_item += "-"
            if item.stop is not None:
                new_item += str(item.stop)
            item = new_item + "]"

        if item in self.__dict__:
            return self.__dict__[item]
        if not self.has_matching_inventory(item):  # type: ignore[no-untyped-call]
            raise KeyError(item)
        self.options["host_pattern"] = item
        return self._dispatcher(**self.options)  # type: ignore[no-untyped-call]

    def __getattr__(self, attr):  # type: ignore[no-untyped-def]  # noqa: ANN001, ANN204
        """Return a ModuleDispatcher instance described the provided `attr`."""
        if not self.has_matching_inventory(attr):  # type: ignore[no-untyped-call]
            msg = f"type HostManager has no attribute '{attr}'"
            raise AttributeError(msg)
        self.options["host_pattern"] = attr
        return self._dispatcher(**self.options)  # type: ignore[no-untyped-call]

    def keys(self):  # type: ignore[no-untyped-def]  # noqa: ANN201
        """Fixme."""
        inventory_hosts = [h.name for h in self.options["inventory_manager"].list_hosts()]
        extra_inventory_hosts = self.get_extra_inventory_hosts()  # type: ignore[no-untyped-call]
        return inventory_hosts + extra_inventory_hosts

    def __iter__(self):  # type: ignore[no-untyped-def]  # noqa: ANN204
        """Return an iterator for hosts matching the `host_pattern`."""
        extra_hosts = self.get_extra_inventory_hosts(  # type: ignore[no-untyped-call]
            host_pattern=self.options["host_pattern"],
        )
        all_hosts = self.options["inventory_manager"].list_hosts(
            self.options["host_pattern"],
        )
        # Return only the name (ala .keys()
        # Return a BaseHostManager instance initialized for each host in the inventory
        # Return a ModuleDispatcher representing the group
        return iter([self[h] for h in all_hosts + extra_hosts])

    def __len__(self) -> int:
        """Return the number of inventory hosts."""
        return len(self.options["inventory_manager"].list_hosts()) + len(
            self.get_extra_inventory_hosts(),  # type: ignore[no-untyped-call]
        )

    def __contains__(self, item) -> bool:  # type: ignore[no-untyped-def]  # noqa: ANN001
        """Return whether there is inventory matching the provided `item`."""
        return self.has_matching_inventory(item)  # type: ignore[no-any-return, no-untyped-call]

    def initialize_inventory(self):  # type: ignore[no-untyped-def]  # noqa: ANN201
        """Fixme."""
        msg = "Must be implemented by sub-class"
        raise NotImplementedError(msg)



"""Tests specific to the molecule plugin functionality."""

from __future__ import annotations

import os
import subprocess
import sys

from typing import TYPE_CHECKING

import pytest


if TYPE_CHECKING:
    from pytest_ansible.molecule import MoleculeScenario


def test_molecule_collect() -> None:
    """Test pytest collection of molecule scenarios."""
    try:
        proc = subprocess.run(
            "pytest --molecule --collect-only",  # noqa: S607
            capture_output=True,
            shell=True,
            check=True,
            text=True,
        )
    except subprocess.CalledProcessError as exc:
        pytest.fail(exc.stderr)

    assert proc.returncode == 0
    assert "test1[default]" in proc.stdout


def test_molecule_disabled() -> None:
    """Ensure the lack of --molecule disables molecule support."""
    proc = subprocess.run(
        f"{sys.executable} -m pytest tests/fixtures/molecule/default/molecule.yml",
        capture_output=True,
        check=False,
        env={"PATH": os.environ["PATH"]},
        shell=True,
        text=True,
    )
    assert proc.returncode == 4  # noqa: PLR2004
    # First check is for pytest 7 behavior, second for pytest >=8
    assert "ERROR: found no collectors" in proc.stderr or "ERROR: not found" in proc.stderr


def test_molecule_runtest() -> None:
    """Test running the molecule scenario via pytest."""
    try:
        proc = subprocess.run(
            f"{sys.executable} -m pytest -v --molecule tests/fixtures/molecule/default/molecule.yml",  # noqa: E501
            capture_output=True,
            check=True,
            env={"PATH": os.environ["PATH"]},
            shell=True,
            text=True,
        )
        assert proc.returncode == 0
        assert "collected 1 item" in proc.stdout
        assert "tests/fixtures/molecule/default/molecule.yml::test" in proc.stdout
        assert "1 passed" in proc.stdout

    except subprocess.CalledProcessError as exc:
        pytest.fail(exc.stderr)


def test_molecule_fixture(molecule_scenario: MoleculeScenario) -> None:
    """Test the scenario fixture.

    :param molecule_scenario: One scenario
    """
    assert molecule_scenario.test_id in ["fixtures-default", "extensions-default"]
    assert molecule_scenario.name == "default"
    molecule_scenario.test()



"""Setup the collection for testing."""

from __future__ import annotations

import logging
import os
import sys

from pathlib import Path


logger = logging.getLogger(__name__)

try:
    import yaml

    HAS_YAML = True
except ImportError:
    HAS_YAML = False

try:
    HAS_ANSIBLE = True
except ImportError:
    HAS_ANSIBLE = False

try:
    from ansible.utils.collection_loader._collection_finder import (
        _AnsibleCollectionFinder,
    )

    HAS_COLLECTION_FINDER = True
except ImportError:
    HAS_COLLECTION_FINDER = False


def get_collection_name(start_path: Path) -> tuple[str | None, str | None]:
    """Get the collection namespace and name from the galaxy.yml file.

    :param start_path: The path to the root of the collection
    :returns: A tuple of the namespace and name
    """
    info_file = start_path / "galaxy.yml"
    logger.info("Looking for collection info in %s", info_file)

    try:
        with info_file.open(encoding="utf-8") as file_handler:
            galaxy_info = yaml.safe_load(file_handler)
    except FileNotFoundError:
        logger.error("No galaxy.yml file found, plugin not activated")  # noqa: TRY400
        return None, None

    try:
        namespace = galaxy_info["namespace"]
        name = galaxy_info["name"]
    except KeyError:
        logger.error("galaxy.yml file does not contain namespace and name")  # noqa: TRY400
        return None, None

    logger.debug("galaxy.yml file found, plugin activated")
    logger.info("Collection namespace: %s", namespace)
    logger.info("Collection name: %s", name)
    return namespace, name


def inject(start_path: Path) -> None:
    """Inject the collection path.

    In the case of ansible > 2.9, initialize the collection finder with the collection path
    otherwise, inject the collection path into sys.path.

    :param start_path: The path where pytest was invoked
    """
    if not HAS_ANSIBLE:
        logger.error("ansible is not installed, plugin not activated")
        return
    if not HAS_YAML:
        logger.error("pyyaml is not installed, plugin not activated")
        return

    logger.debug("Start path: %s", start_path)
    namespace, name = get_collection_name(start_path)
    if namespace is None or name is None:
        # Tests may not being run from the root of the repo.
        return

    # Determine if the start_path is in a collections tree
    collection_tree = ("collections", "ansible_collections", namespace, name)
    if start_path.parts[-4:] == collection_tree:
        logger.info("In collection tree")
        collections_dir = start_path.parents[2]

    else:
        logger.info("Not in collection tree")
        collections_dir = start_path / "collections"
        name_dir = collections_dir / "ansible_collections" / namespace / name

        # If it's here, we will trust it was from this
        if not name_dir.is_dir():
            name_dir.mkdir(parents=True, exist_ok=True)

            for entry in start_path.iterdir():
                if entry.name == "collections":
                    continue
                os.symlink(entry, name_dir / entry.name)

    # Configuration option for additional collection paths
    additional_collections_paths = [Path("~/.ansible/collections").expanduser()]

    # Check if the environment variable is set for additional paths
    if "COLLECTIONS_PATH" in os.environ and "COLLECTIONS_PATHS" in os.environ:
        additional_collections_paths.extend(
            os.environ.get("COLLECTIONS_PATH", "").split(os.pathsep)  # type: ignore[arg-type]
            + os.environ.get("COLLECTIONS_PATHS", "").split(os.pathsep),
        )
    logger.info("Additional Collections Paths: %s", additional_collections_paths)

    acf_inject(paths=[str(collections_dir), *additional_collections_paths])  # type: ignore[list-item]

    # Inject the path for the collection into sys.path
    sys.path.insert(0, str(collections_dir))
    logger.debug("sys.path updated: %s", sys.path)

    # Set the environment variable as a courtesy for integration tests
    envvar_name = determine_envvar()
    # Assuming additional_collections_paths is a list of PosixPath objects
    additional_collections_paths = [str(path) for path in additional_collections_paths]  # type: ignore[misc]
    env_paths = os.pathsep.join([str(collections_dir), *additional_collections_paths])  # type: ignore[list-item]
    logger.info("Setting %s to %s", envvar_name, env_paths)
    os.environ[envvar_name] = env_paths


def inject_only() -> None:
    """Inject the current ANSIBLE_COLLECTIONS_PATH(S)."""
    envvar_name = determine_envvar()
    env_paths = os.environ.get(envvar_name, "")
    path_list = env_paths.split(os.pathsep)
    for path in path_list:
        if path:
            sys.path.insert(0, path)
    logger.debug("sys.path updated: %s", sys.path)
    acf_inject(paths=path_list)


def acf_inject(paths: list[str]) -> None:
    """Inject the collection path into the AnsibleCollectionFinder.

    :param paths: The paths to inject
    """
    if HAS_COLLECTION_FINDER:
        acf = _AnsibleCollectionFinder(paths=paths)
        acf._install()  # noqa: SLF001
        logger.debug("_ACF installed: %s", paths)
        logger.debug("_ACF configured paths: %s", acf._n_configured_paths)  # noqa: SLF001
    else:
        logger.debug("_ACF not available")


def determine_envvar() -> str:
    """Use the existence of the AnsibleCollectionFinder to determine the ansible version.

    Ansible 2.9 did not have AnsibleCollectionFinder and did not support ANSIBLE_COLLECTIONS_PATH later versions do.

    :returns: The appropriate environment variable to use
    """  # noqa: E501
    if not HAS_COLLECTION_FINDER:
        return "ANSIBLE_COLLECTIONS_PATHS"
    return "ANSIBLE_COLLECTIONS_PATH"



"""Host manager related utilities."""

from __future__ import annotations

from typing import TYPE_CHECKING

from pytest_ansible.has_version import has_ansible_v213


if TYPE_CHECKING:
    from pytest_ansible.host_manager.base import BaseHostManager


def get_host_manager(*args, **kwargs) -> BaseHostManager:  # type: ignore[no-untyped-def]  # noqa: ANN002, ANN003
    """Initialize and return a HostManager instance.

    Args:
        *args: Positional arguments to pass to the HostManager constructor.
        **kwargs: Keyword arguments to pass to the HostManager constructor.

    Returns:
        HostManager: A HostManager instance.
    """
    if has_ansible_v213:
        from pytest_ansible.host_manager.v213 import HostManagerV213 as HostManager
    else:
        err = "Unable to find any supported HostManager"
        raise RuntimeError(err)

    return HostManager(*args, **kwargs)



"""Adhoc result tests."""

from __future__ import annotations

from types import GeneratorType

import pytest

from pytest_ansible.results import ModuleResult

from .conftest import ALL_EXTRA_HOSTS, ALL_HOSTS


invalid_hosts = [
    pytest.param("none"),
    pytest.param("all"),
    pytest.param("*", id="glob"),
    pytest.param("local*", id="glob2"),
]


@pytest.fixture(params=[True, False], name="adhoc_result")
def fixture_adhoc_result(request, hosts):  # type: ignore[no-untyped-def]  # noqa: ANN001, ANN201, D103
    def create_hosts():  # type: ignore[no-untyped-def]  # noqa: ANN202
        _hosts = hosts(include_extra_inventory=request.param)
        return _hosts.all.ping(), request.param

    return create_hosts


def test_len(adhoc_result):  # type: ignore[no-untyped-def]  # noqa: ANN001, ANN201, D103
    adhoc_result_ret, include_extra_inv = adhoc_result()
    assert len(adhoc_result_ret) == len(ALL_HOSTS) + len(
        ALL_EXTRA_HOSTS if include_extra_inv else [],
    )


def test_keys(adhoc_result):  # type: ignore[no-untyped-def]  # noqa: ANN001, ANN201, D103
    adhoc_result_ret, include_extra_inv = adhoc_result()
    assert set(adhoc_result_ret) == set(
        ALL_HOSTS + (ALL_EXTRA_HOSTS if include_extra_inv else []),
    )


def test_items(adhoc_result):  # type: ignore[no-untyped-def]  # noqa: ANN001, ANN201, D103
    adhoc_result_ret, include_extra_inv = adhoc_result()
    items = adhoc_result_ret.items()
    assert isinstance(items, GeneratorType)
    count = 0
    for count, item in enumerate(items, 1):  # noqa: B007
        assert isinstance(item, tuple)
        assert isinstance(item[0], str)
        assert isinstance(item[1], ModuleResult)
    assert count == len(ALL_HOSTS + (ALL_EXTRA_HOSTS if include_extra_inv else []))


def test_values(adhoc_result):  # type: ignore[no-untyped-def]  # noqa: ANN001, ANN201, D103
    adhoc_result_ret, include_extra_inv = adhoc_result()
    values = adhoc_result_ret.values()
    assert isinstance(values, list)
    # assure that it is a copy
    assert values is not adhoc_result_ret.contacted.values()
    count = 0
    for count, val in enumerate(values, 1):  # noqa: B007
        assert isinstance(val, ModuleResult)
    assert count == len(ALL_HOSTS) + len(ALL_EXTRA_HOSTS if include_extra_inv else [])


@pytest.mark.parametrize("host", ALL_HOSTS + ALL_EXTRA_HOSTS)
def test_contains(adhoc_result, host):  # type: ignore[no-untyped-def]  # noqa: ANN001, ANN201, D103
    adhoc_result_ret, include_extra_inv = adhoc_result()
    if not include_extra_inv and host in ALL_EXTRA_HOSTS:
        assert host not in adhoc_result_ret
    else:
        assert host in adhoc_result_ret


@pytest.mark.parametrize("host", invalid_hosts)
def test_not_contains(adhoc_result, host):  # type: ignore[no-untyped-def]  # noqa: ANN001, ANN201, D103
    adhoc_result_ret, dummy = adhoc_result()
    assert host not in adhoc_result_ret


@pytest.mark.parametrize("host_pattern", ALL_HOSTS + ALL_EXTRA_HOSTS)
def test_getitem(adhoc_result, host_pattern):  # type: ignore[no-untyped-def]  # noqa: ANN001, ANN201, D103
    adhoc_result_ret, include_extra_inv = adhoc_result()
    if not include_extra_inv and host_pattern in ALL_EXTRA_HOSTS:
        with pytest.raises(KeyError):
            assert adhoc_result_ret[host_pattern]
    else:
        assert adhoc_result_ret[host_pattern]
        assert isinstance(adhoc_result_ret[host_pattern], ModuleResult)


@pytest.mark.parametrize("host_pattern", invalid_hosts)
def test_not_getitem(adhoc_result, host_pattern):  # type: ignore[no-untyped-def]  # noqa: ANN001, ANN201, D103
    adhoc_result_ret, dummy = adhoc_result()
    with pytest.raises(KeyError):
        assert adhoc_result_ret[host_pattern]


@pytest.mark.parametrize("host_pattern", ALL_HOSTS + ALL_EXTRA_HOSTS)
def test_getattr(adhoc_result, host_pattern):  # type: ignore[no-untyped-def]  # noqa: ANN001, ANN201, D103
    adhoc_result_ret, include_extra_inv = adhoc_result()
    if not include_extra_inv and host_pattern in ALL_EXTRA_HOSTS:
        assert not hasattr(adhoc_result_ret, host_pattern)
    else:
        assert hasattr(adhoc_result_ret, host_pattern)
        assert isinstance(adhoc_result_ret[host_pattern], ModuleResult)


@pytest.mark.parametrize("host_pattern", invalid_hosts)
def test_not_getattr(adhoc_result, host_pattern):  # type: ignore[no-untyped-def]  # noqa: ANN001, ANN201, D103
    adhoc_result_ret, dummy = adhoc_result()
    assert not hasattr(adhoc_result_ret, host_pattern)
    with pytest.raises(AttributeError):
        getattr(adhoc_result_ret, host_pattern)


@pytest.mark.requires_ansible_v2()
def test_connection_failure_v2():  # type: ignore[no-untyped-def]  # noqa: ANN201, D103
    from pytest_ansible.errors import AnsibleConnectionFailure
    from pytest_ansible.host_manager.utils import get_host_manager

    hosts = get_host_manager(inventory="unknown.example.com,", connection="smart")
    with pytest.raises(AnsibleConnectionFailure) as exc_info:
        hosts.all.ping()
    # Assert message
    assert exc_info.value.message == "Host unreachable in the inventory"
    # Assert contacted
    assert exc_info.value.contacted == {}
    # Assert dark
    assert "unknown.example.com" in exc_info.value.dark
    # Assert unreachable
    assert "unreachable" in exc_info.value.dark["unknown.example.com"], exc_info.value.dark.keys()
    assert exc_info.value.dark["unknown.example.com"]["unreachable"]
    # Assert msg
    assert "msg" in exc_info.value.dark["unknown.example.com"]
    assert (
        "Failed to connect to the host via ssh" in exc_info.value.dark["unknown.example.com"]["msg"]
    )


@pytest.mark.requires_ansible_v2()
def test_connection_failure_extra_inventory_v2():  # type: ignore[no-untyped-def]  # noqa: ANN201, D103
    from pytest_ansible.errors import AnsibleConnectionFailure
    from pytest_ansible.host_manager.utils import get_host_manager

    hosts = get_host_manager(
        inventory="localhost",
        extra_inventory="unknown.example.extra.com,",
    )
    with pytest.raises(AnsibleConnectionFailure) as exc_info:
        hosts.all.ping()
    # Assert message
    assert exc_info.value.message == "Host unreachable in the extra inventory"
    # Assert contacted
    assert exc_info.value.contacted == {}
    # Assert dark
    assert "unknown.example.extra.com" in exc_info.value.dark
    # Assert unreachable
    assert (
        "unreachable" in exc_info.value.dark["unknown.example.extra.com"]
    ), exc_info.value.dark.keys()
    assert exc_info.value.dark["unknown.example.extra.com"]["unreachable"]
    # Assert msg
    assert "msg" in exc_info.value.dark["unknown.example.extra.com"]
    assert (
        "Failed to connect to the host via ssh"
        in exc_info.value.dark["unknown.example.extra.com"]["msg"]
    )
